@@ -1,4 +1,4 @@
-/*  $Id:$
+/*  $Id: xfcesmart.c 3315 2007-10-08 08:53:04Z afb $
  *
  *  xfce4-smartpm-plugin - a package manager applet for the xfce4 panel
  *  Copyright (c) 2007 Anders F Bjorklund <afb@users.sourceforge.net>
@@ -298,14 +298,14 @@ smart_update_channels( SmartPlugin    *smart)
       terminal = FALSE;
       break;
     case PACKAGE_MANAGER_YUM:
-      argv[0] = "/bin/su";
+      argv[0] = PATH_SU;
       argv[1] = "-c";
       argv[2] = "\"" YUM_UPDATE " "\
                 "check-update" "\""; /* yum doesn't have a method to just update */
       terminal = TRUE;
       break;
     case PACKAGE_MANAGER_APT:
-      argv[0] = "/usr/bin/sudo";
+      argv[0] = PATH_SUDO;
       argv[1] = APT_UPDATE;
       argv[2] = "update";
       terminal = TRUE;
@@ -481,15 +481,15 @@ smart_check_upgrades_watch_func(GPid pid, gint status, gpointer user_data)
   {
     while ( (p = fgets(buf, sizeof(buf), file)) != NULL )
     {
-      if (strstr(p, "Failed to write temporary StateFile") != NULL &&
-          smart->package_manager == PACKAGE_MANAGER_APT)
+      if (smart->package_manager == PACKAGE_MANAGER_APT &&
+          strstr(p, "Failed to write temporary StateFile") != NULL)
       {
         /* skip this error, we don't want to write anything anyway */
         status = 0;
         continue;
       }
-      if (strstr(p, "error: ") != NULL ||
-          smart->package_manager != PACKAGE_MANAGER_SMART)
+      if (smart->package_manager != PACKAGE_MANAGER_SMART ||
+          strstr(p, "error: ") != NULL)
       {
         if (strstr(p, "error: ") == 0)
           p += strlen("error: ");
@@ -578,11 +578,11 @@ smart_check_upgrades( SmartPlugin    *smart)
       argv[2] = "check-update";
       break;
     case PACKAGE_MANAGER_APT:
+      tempfile = g_strdup_printf("%s%s%s-apt", g_get_tmp_dir(), G_DIR_SEPARATOR_S, g_get_user_name());
       argv[0] = APT_UPDATE;
       argv[1] = "--simulate";
-      argv[2] = "dist-upgrade";
+      argv[2] = (smart->dist_upgrade) ? "dist-upgrade" : "upgrade";
       argv[3] = "--option"; /* regular cache files is only writable by root */
-      tempfile = g_strdup_printf("%s%s%s-apt", g_get_tmp_dir(), G_DIR_SEPARATOR_S, g_get_user_name());
       argv[4] = pkgcache = g_strdup_printf("Dir::Cache::pkgcache=%spkg.cache", tempfile);
       argv[5] = "--option"; /* regular cache files is only writable by root */
       argv[6] = srcpkgcache = g_strdup_printf("Dir::Cache::srcpkgcache=%ssrc.cache", tempfile);
@@ -718,6 +718,7 @@ smart_save (XfcePanelPlugin *plugin,
       /* save the settings */
       xfce_rc_write_int_entry  (rc, "interval", smart->check_interval);
       xfce_rc_write_bool_entry (rc, "upgrade", smart->upgrade_directly);
+      xfce_rc_write_bool_entry (rc, "dist-upgrade", smart->dist_upgrade);
       xfce_rc_write_int_entry (rc, "manager", smart->package_manager);
       xfce_rc_write_bool_entry (rc, "appicon", smart->show_application);
 
@@ -750,6 +751,7 @@ smart_read (SmartPlugin *smart)
           /* read the settings */
           smart->check_interval = xfce_rc_read_int_entry (rc, "interval", 0);
           smart->upgrade_directly =  xfce_rc_read_bool_entry (rc, "upgrade", TRUE);
+          smart->dist_upgrade =  xfce_rc_read_bool_entry (rc, "dist-upgrade", TRUE);
           smart->package_manager = xfce_rc_read_int_entry (rc, "manager", PACKAGE_MANAGER_SMART);
           smart->show_application =  xfce_rc_read_bool_entry (rc, "appicon", TRUE);
 
@@ -766,6 +768,7 @@ smart_read (SmartPlugin *smart)
 
   smart->check_interval = 0;
   smart->upgrade_directly = TRUE;
+  smart->dist_upgrade = TRUE;
   smart->package_manager = PACKAGE_MANAGER_SMART;
   smart->show_application = TRUE;
 }
@@ -846,12 +849,18 @@ smart_combobox_changed_cb(GtkComboBox *combo,
         gpointer user_data)
 {
   SmartPlugin *smart = user_data;
+  GtkWidget *button;
   gint manager;
 
   manager = gtk_combo_box_get_active(combo);
 
   smart_set_manager (smart, manager);
 
+  button = g_object_get_data (G_OBJECT (smart->plugin), "dist-upgrade");
+
+  if (button != NULL)
+  gtk_widget_set_sensitive (button, smart->package_manager == PACKAGE_MANAGER_APT);
+
   if (smart->show_application) /* change the application icon too */
   smart_size_changed (smart->plugin, smart->current_icon_size, smart);
 }
@@ -894,6 +903,15 @@ smart_checkbox2_toggle_cb(GtkToggleButton *button,
   smart->upgrade_directly = gtk_toggle_button_get_active(button);
 }
 
+static void
+smart_checkbox3_toggle_cb(GtkToggleButton *button,
+        gpointer user_data)
+{
+  SmartPlugin *smart = user_data;
+
+  smart->dist_upgrade = gtk_toggle_button_get_active(button);
+}
+
 static SmartPlugin *
 smart_new (XfcePanelPlugin *plugin)
 {
@@ -1165,6 +1183,8 @@ smart_configure_response (GtkWidget    *dialog,
     }
   else if (response == GTK_RESPONSE_OK)
     {
+      g_object_set_data (G_OBJECT (smart->plugin), "dist-upgrade", NULL);
+
       /* remove the dialog data from the plugin */
       g_object_set_data (G_OBJECT (smart->plugin), "dialog", NULL);
       
@@ -1303,6 +1323,15 @@ smart_configure (XfcePanelPlugin *plugin,
   g_signal_connect (G_OBJECT(button),"toggled", G_CALLBACK(smart_checkbox2_toggle_cb), smart);
   gtk_box_pack_start (GTK_BOX(vbox),button,TRUE,TRUE,0);
 
+  button = gtk_check_button_new_with_label(_("Smart upgrade to newer release (dist-upgrade)"));
+  gtk_widget_show (button);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), smart->dist_upgrade);
+  g_signal_connect (G_OBJECT(button),"toggled", G_CALLBACK(smart_checkbox3_toggle_cb), smart);
+  gtk_box_pack_start (GTK_BOX(vbox),button,TRUE,TRUE,0);
+
+  g_object_set_data (G_OBJECT (plugin), "dist-upgrade", button);
+  gtk_widget_set_sensitive (button, smart->package_manager == PACKAGE_MANAGER_APT);
+
   /* center dialog on the screen */
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 
